///
/// @file         update.h
/// @copyright    Copyright (c) 2015 Toradex AG
/// $Author: andy.kiser $
/// $Revision: 0000 $
/// $Date: 2015-08-12 15:53:09 +0200 (Mi, 12 Aug 2015) $
/// @brief        Library to support updating bootloader and OS on Toradex modules.
/// @test         Colibri VF61 and VF50
/// @target       all Colibri and Apalis modules.
///
///
/// @anchor update_parameters
/// Update Library for Toradex modules. 
/// * Update flash memory sections from a file
/// * Backup flash memory sections to a file
///
/// @page pcfg_update Update Configuration Options
/// @tableofcontents
///
/// @section scfg_upd Compatible Update Configurations
/// @anchor  acfg_upd
/// The following subset of Update configuration parameters is expected to be
/// available on any SoC.\n
/// Toradex  recommends to prefer these settings over the SoC specific
/// configurations. This can simplify the porting to any other Toradex hardware.
///
/// <table class="libParam" >
///   <tr><th>              Parameter Name</th><th>Value   </th><th> Description</th></tr>
///   <tr><td class="paramname"          >filename  </td><td>STRING  </td><td>Full path to the file to read write the backup to, or to read the update information from.</td></tr>
///   <tr><td class="paramname"          >fileformat</td><td>STRING  </td><td>The file format used. Refer to the SoC specific documentation for details about the supported formats.</td></tr>
///   <tr><td class="paramname"          >sections  </td><td>STRING  </td><td>A comma-separated list of flash sections that will be backuped to the file, or restored from the file.<br>Refer to the SoC specific documentation for a full list of supported sections.<br>Commonly supported sections on all SoCs are<br><ul><li>bootloader<li>os<li>configblock</ul>Spcific sections can include<ul><li>guess<li>bootloader2<li>os2<li>splashscreen<li>filesystem<li>registry<li>bct</ul></td></tr>
///   <tr><td class="paramname"          >callback  </td><td>DWORD   </td><td>Optional pointer to a callback function.\n The callback function is used to report the progress of the update procedure and has to be a prototype defined as type @ref Upd_Callback.</td></tr>
/// </table>

#ifndef _UPDATE_H_
#define _UPDATE_H_

#include "TdxCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

//******************************************************************************
/// Type definition for a callback function.\n
/// This function will be called from withing Upd_Backup() and Upd_Restore() to indicate the overall procss of the backup / update procedure\n
/// @param[in]  progress    Overall progress of the Backup or Restore procedure (0 = 0% ... 65536 = 100%)\n 
///                         The library default is to call no callback function. If you want to have one, you need to register your function by calling Upd_SetConfigString().
typedef void (*Upd_Callback)(DWORD progress);
//******************************************************************************


//******************************************************************************
/// Initialize HANDLE for the Update Process.\n
/// @param[in]  registryPath    Registry path string value.e.g \c SOFTWARE\\Toradex\\Update.\n
///                             We placed this parameter here, even if we currently don't see any usage for registry settings for the Update Library.
/// @return                     Handle to the Update Process. NULL if the function failed.
HANDLE Upd_Init(const TCHAR *registryPath);

//******************************************************************************
/// De-initialize HANDLE for the Update Process.\n
/// @param[in]    hUpdate       HANDLE received from Upd_Init()
/// @retval       TRUE          Success
/// @retval       FALSE         Failure
BOOL Upd_Deinit(HANDLE hUpdate);

//******************************************************************************
// Open the a file to backup to, or to update from.\n
// Refer to the SoC specific documentation to learn details about the file format.
// The filename and sections need to be setup first through calls to Upd_SetConfigString().
// @param[in]  hUpdate         HANDLE received from Upd_Init()
// @retval     TRUE            Success
// @retval     FALSE           Failure
//BOOL Upd_Open(HANDLE hUpdate);
// We decided to remove Open() and Close() functions - Init() and Deinit() are sufficient

//******************************************************************************
// Close Update process.
// @param[in]  hUpdate         HANDLE received from Upd_Init()
// @retval     TRUE            Success
// @retval     FALSE           Failure
//BOOL Upd_Close(HANDLE hUpdate);
// We decided to remove Open() and Close() functions - Init() and Deinit() are sufficient

//******************************************************************************
/// Reads data from the module's flash storage, and stores it to a file.\n
/// The filename and sections need to be setup first through calls to Upd_SetConfigString().\n
/// Refer to the SoC specific documentation to learn details about the file format.
/// @param[in]  hUpdate         HANDLE received from Upd_Init()
/// @return     Number of Bytes backed up.
/// @retval     FALSE           Failure
DWORD Upd_Backup(HANDLE hUpdate);

//******************************************************************************
/// Read data from a file and restore it to the device's flash memory.\n
/// The filename and sections need to be setup first through calls to Upd_SetConfigString().\n
/// Refer to the SoC specific documentation to learn details about the file format.
/// @param[in]  hUpdate         HANDLE received from Upd_Init()
/// @return                     Number of Bytes restored
DWORD Upd_Restore(HANDLE hUpdate);

//******************************************************************************
/// Set a STRING configuration parameter.\n
/// NOTE: For security reasons, all previously configured parameters are
///       invalidated, if any error happens while executing this function!
/// @param[in]  hUpdate         HANDLE received from Upd_Init()
/// @param[in]  paramName       Parameter value to configure.\n
///                             Refer to @ref scfg_upd for details.
/// @param[in]  value           String value to be set
/// @param[in]  storageType     StoreInRegistry = parameter will also be stored in the registry\n
///                             StoreVolatile   = parameter will only be stored in local variable.
/// @retval     TRUE            Success
/// @retval     FALSE           Failure
BOOL Upd_SetConfigString(HANDLE hUpdate, const TCHAR *paramName, const TCHAR *value, ParamStorageType storageType);

//******************************************************************************
/// Set a DWORD configuration parameter.\n
/// NOTE: For security reasons, all previously configured parameters are
///       invalidated, if any error happens while executing this function!
/// @param[in]  hUpdate         HANDLE received from Upd_Init()
/// @param [in] paramName       Parameter value to configure.\n
///                             Refer to @ref scfg_upd for details.
/// @param [in] value           Data to be set
/// @param[in]  storageType     StoreInRegistry = parameter will also be stored in the registry\n
///                             StoreVolatile   = parameter will only be stored in local variable.
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Upd_SetConfigInt(HANDLE hUpdate, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Retrieves a STRING configuration parameter.
/// @param[in]   hUpdate        HANDLE received from Upd_Init()
/// @param[in]   paramName      Parameter value to retrieve.\n
///                             Refer to @ref scfg_upd for details.
/// @param[out]  pValue         String data obtained
/// @param[in]   maxBytes       Maximum size of string in Bytes
/// @return      Number of bytes read. 0 indicates an error.
DWORD Upd_GetConfigString(HANDLE hUpdate, const TCHAR *paramName, TCHAR *pValue, DWORD maxBytes);

//******************************************************************************
/// Retrieves a DWORD configuration parameter.
/// @param[in]   hUpdate        HANDLE received from Upd_Init()
/// @param[in]   paramName      Parameter value to retrieve.\n
///                             Refer to @ref scfg_upd for details.
/// @param[out]  pValue         DWORD data obtained
/// @return      Number of bytes read. 0 indicates an error.
DWORD Upd_GetConfigInt(HANDLE hUpdate, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Save the registry changes permanently to flash.\n
/// On some OS implementations it is possible to preselect saving the system part,
/// the user part or both parts from the registry (see @ref scfg_upd)
/// @param[in]   hUpdate        HANDLE received from Upd_Init()
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Upd_RegistrySave(HANDLE hUpdate);

//******************************************************************************
/// Delete the flash registry information in the flash, i.e. restore the
/// factory-default registry settings.\n
/// On some OS implementations it is possible to preselect clearing the system part,
/// the user part or both parts from the registry (see @ref scfg_upd)
/// @param[in]   hUpdate        HANDLE received from Upd_Init()
/// @retval      TRUE           Success
/// @retval      FALSE          Failure
BOOL Upd_RegistryClear(HANDLE hUpdate);

//******************************************************************************
/// Get the revision of this Update library
/// @param[out]   ver           library version, consisting of major, minor and build version
void Upd_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif //_UPDATE_H_
